@@ -598,6 +598,17 @@ class TestIniParser:
         assert argvlist[2] == ['cmd2', '-m', "something"] + posargs
         assert argvlist[3] == ['cmd3', 'something[]else']
 
+    def test_posargs_are_added_escaped_issue310(self, newconfig):
+        config = newconfig("""
+            [section]
+            key= cmd0 {posargs}
+        """)
+        reader = SectionReader("section", config._cfg)
+        posargs = ['hello world', '--x==y z', '--format=%(code)s: %(text)s']
+        reader.addsubstitutions(posargs)
+        argvlist = reader.getargvlist('key')
+        assert argvlist[0] == ['cmd0'] + posargs
+
     def test_substitution_with_multiple_words(self, newconfig):
         inisource = """
             [section]
@@ -835,6 +846,7 @@ class TestConfigTestEnv:
             assert "PATHEXT" in envconfig.passenv
             assert "SYSTEMDRIVE" in envconfig.passenv
             assert "SYSTEMROOT" in envconfig.passenv
+            assert "COMSPEC" in envconfig.passenv
             assert "TEMP" in envconfig.passenv
             assert "TMP" in envconfig.passenv
         else:
@@ -980,31 +992,6 @@ class TestConfigTestEnv:
         envconfig = config.envconfigs['python']
         assert envconfig.pip_pre
 
-    def test_downloadcache(self, newconfig, monkeypatch):
-        monkeypatch.delenv("PIP_DOWNLOAD_CACHE", raising=False)
-        config = newconfig("""
-            [testenv]
-            downloadcache=thecache
-        """)
-        envconfig = config.envconfigs['python']
-        assert envconfig.downloadcache.basename == 'thecache'
-
-    def test_downloadcache_env_override(self, newconfig, monkeypatch):
-        monkeypatch.setenv("PIP_DOWNLOAD_CACHE", 'fromenv')
-        config = newconfig("""
-            [testenv]
-            downloadcache=somepath
-        """)
-        envconfig = config.envconfigs['python']
-        assert envconfig.downloadcache.basename == "fromenv"
-
-    def test_downloadcache_only_if_in_config(self, newconfig, tmpdir,
-                                             monkeypatch):
-        monkeypatch.setenv("PIP_DOWNLOAD_CACHE", tmpdir)
-        config = newconfig('')
-        envconfig = config.envconfigs['python']
-        assert not envconfig.downloadcache
-
     def test_simple(tmpdir, newconfig):
         config = newconfig("""
             [testenv:py26]
@@ -1468,7 +1455,7 @@ class TestGlobalOptions:
         assert str(env.basepython) == sys.executable
 
     def test_default_environments(self, tmpdir, newconfig, monkeypatch):
-        envs = "py26,py27,py32,py33,py34,py35,py36,jython,pypy,pypy3"
+        envs = "py26,py27,py32,py33,py34,py35,py36,py37,jython,pypy,pypy3,py2,py3"
         inisource = """
             [tox]
             envlist = %s
@@ -1482,6 +1469,8 @@ class TestGlobalOptions:
                 assert env.basepython == "jython"
             elif name.startswith("pypy"):
                 assert env.basepython == name
+            elif name in ("py2", "py3"):
+                assert env.basepython == 'python' + name[-1]
             else:
                 assert name.startswith("py")
                 bp = "python%s.%s" % (name[2], name[3])
@@ -1874,6 +1863,28 @@ class TestIndexServer:
         assert config.indexserver['local1'].url == config.indexserver['default'].url
 
 
+class TestConfigConstSubstitutions:
+    @pytest.mark.parametrize('pathsep', [':', ';'])
+    def test_replace_pathsep_unix(self, monkeypatch, newconfig, pathsep):
+        monkeypatch.setattr('os.pathsep', pathsep)
+        config = newconfig("""
+        [testenv]
+        setenv =
+            PATH = dira{:}dirb{:}dirc
+        """)
+        envconfig = config.envconfigs["python"]
+        assert envconfig.setenv["PATH"] == pathsep.join(["dira", "dirb", "dirc"])
+
+    def test_pathsep_regex(self):
+        """Sanity check for regex behavior for empty colon."""
+        regex = tox.config.Replacer.RE_ITEM_REF
+        match = next(regex.finditer("{:}"))
+        mdict = match.groupdict()
+        assert mdict['sub_type'] is None
+        assert mdict['substitution_value'] == ""
+        assert mdict['default_value'] == ""
+
+
 class TestParseEnv:
 
     def test_parse_recreate(self, newconfig):
