@@ -199,6 +199,34 @@ def test_minversion(cmd, initproj):
     assert result.ret
 
 
+def test_notoxini_help_still_works(initproj, cmd):
+    initproj("example123-0.5", filedefs={
+        'tests': {'test_hello.py': "def test_hello(): pass"},
+    })
+    result = cmd.run("tox", "-h")
+    result.stderr.fnmatch_lines([
+        "*ERROR*tox.ini*"
+    ])
+    result.stdout.fnmatch_lines([
+        "*--help*"
+    ])
+    assert not result.ret
+
+
+def test_notoxini_help_ini_still_works(initproj, cmd):
+    initproj("example123-0.5", filedefs={
+        'tests': {'test_hello.py': "def test_hello(): pass"},
+    })
+    result = cmd.run("tox", "--help-ini")
+    result.stderr.fnmatch_lines([
+        "*ERROR*tox.ini*"
+    ])
+    result.stdout.fnmatch_lines([
+        "*setenv*"
+    ])
+    assert not result.ret
+
+
 def test_envdir_equals_toxini_errors_out(cmd, initproj):
     initproj("interp123-0.7", filedefs={
         'tox.ini': '''
@@ -323,6 +351,8 @@ def test_venv_special_chars_issue252(cmd, initproj):
         'tox.ini': '''
             [tox]
             envlist = special&&1
+            [testenv]
+            commands={envpython} --version
             [testenv:special&&1]
             changedir=tests
         '''
@@ -440,7 +470,10 @@ def test_package_install_fails(cmd, initproj):
                 install_requires=['qweqwe123'],
                 )
             """,
-        'tox.ini': '',
+        'tox.ini': """
+        [testenv]
+        commands={envpython} --version
+        """,
     })
     result = cmd.run("tox", )
     assert result.ret
@@ -513,6 +546,8 @@ class TestToxRun:
 
 def test_develop(initproj, cmd):
     initproj("example123", filedefs={'tox.ini': """
+        [testenv]
+        commands={envpython} --version
     """})
     result = cmd.run("tox", "-vv", "--develop")
     assert not result.ret
@@ -522,6 +557,7 @@ def test_develop(initproj, cmd):
 def test_usedevelop(initproj, cmd):
     initproj("example123", filedefs={'tox.ini': """
             [testenv]
+            commands={envpython} --version
             usedevelop=True
     """})
     result = cmd.run("tox", "-vv")
@@ -531,6 +567,8 @@ def test_usedevelop(initproj, cmd):
 
 def test_usedevelop_mixed(initproj, cmd):
     initproj("example123", filedefs={'tox.ini': """
+            [testenv]
+            commands={envpython} --version
             [testenv:devenv]
             usedevelop=True
             [testenv:nondev]
@@ -620,6 +658,7 @@ def test_notest(initproj, cmd):
     initproj("example123", filedefs={'tox.ini': """
         # content of: tox.ini
         [testenv:py26]
+        commands={envpython} --version
         basepython=python
     """})
     result = cmd.run("tox", "-v", "--notest")
@@ -636,7 +675,10 @@ def test_notest(initproj, cmd):
 
 
 def test_PYC(initproj, cmd, monkeypatch):
-    initproj("example123", filedefs={'tox.ini': ''})
+    initproj("example123", filedefs={'tox.ini': """
+        [testenv]
+        commands={envpython} --version
+    """})
     monkeypatch.setenv("PYTHONDOWNWRITEBYTECODE", 1)
     result = cmd.run("tox", "-v", "--notest")
     assert not result.ret
@@ -646,7 +688,10 @@ def test_PYC(initproj, cmd, monkeypatch):
 
 
 def test_env_VIRTUALENV_PYTHON(initproj, cmd, monkeypatch):
-    initproj("example123", filedefs={'tox.ini': ''})
+    initproj("example123", filedefs={'tox.ini': """
+        [testenv]
+        commands={envpython} --version
+    """})
     monkeypatch.setenv("VIRTUALENV_PYTHON", '/FOO')
     result = cmd.run("tox", "-v", "--notest")
     assert not result.ret, result.stdout.lines
@@ -672,6 +717,8 @@ def test_separate_sdist_no_sdistfile(cmd, initproj):
         'tox.ini': """
             [tox]
             distshare=%s
+            [testenv]
+            commands={envpython} --version
         """ % distshare
     })
     result = cmd.run("tox", "--sdistonly")
@@ -689,6 +736,8 @@ def test_separate_sdist(cmd, initproj):
             [tox]
             distshare=%s
             sdistsrc={distshare}/pkg123-0.7.zip
+            [testenv]
+            commands={envpython} --version
         """ % distshare
     })
     result = cmd.run("tox", "--sdistonly")
@@ -709,6 +758,8 @@ def test_sdist_latest(tmpdir, newconfig):
             [tox]
             distshare=%s
             sdistsrc={distshare}/pkg123-*
+            [testenv]
+            commands={envpython} --version
     """ % distshare)
     p = distshare.ensure("pkg123-1.4.5.zip")
     distshare.ensure("pkg123-1.4.5a1.zip")
@@ -756,6 +807,23 @@ def test_envsitepackagesdir_skip_missing_issue280(cmd, initproj):
     """)
 
 
+@pytest.mark.parametrize('verbosity', ['', '-v', '-vv'])
+def test_verbosity(cmd, initproj, verbosity):
+    initproj("pkgX-0.0.5", filedefs={
+        'tox.ini': """
+        [testenv]
+        commands={envpython} --version
+    """})
+    result = cmd.run("tox", verbosity)
+    assert result.ret == 0
+
+    needle = "Successfully installed pkgX-0.0.5"
+    if verbosity == '-vv':
+        assert any(needle in line for line in result.outlines)
+    else:
+        assert all(needle not in line for line in result.outlines)
+
+
 def verify_json_report_format(data, testenvs=True):
     assert data["reportversion"] == "1"
     assert data["toxversion"] == tox.__version__
@@ -773,3 +841,30 @@ def verify_json_report_format(data, testenvs=True):
                 assert isinstance(pyinfo["version_info"], list)
                 assert pyinfo["version"]
                 assert pyinfo["executable"]
+
+
+def test_envtmpdir(initproj, cmd):
+    initproj("foo", filedefs={
+        # This file first checks that envtmpdir is existent and empty. Then it
+        # creates an empty file in that directory.  The tox command is run
+        # twice below, so this is to test whether the directory is cleared
+        # before the second run.
+        'check_empty_envtmpdir.py': '''if True:
+            import os
+            from sys import argv
+            envtmpdir = argv[1]
+            assert os.path.exists(envtmpdir)
+            assert os.listdir(envtmpdir) == []
+            open(os.path.join(envtmpdir, 'test'), 'w').close()
+        ''',
+        'tox.ini': '''
+            [testenv]
+            commands=python check_empty_envtmpdir.py {envtmpdir}
+        '''
+    })
+
+    result = cmd.run("tox")
+    assert not result.ret
+
+    result = cmd.run("tox")
+    assert not result.ret
