@@ -9,9 +9,11 @@ import string
 import pkg_resources
 import itertools
 import pluggy
+from subprocess import list2cmdline
 
 import tox.interpreters
 from tox import hookspecs
+from tox._verlib import NormalizedVersion
 
 import py
 
@@ -20,20 +22,27 @@ import tox
 iswin32 = sys.platform == "win32"
 
 default_factors = {'jython': 'jython', 'pypy': 'pypy', 'pypy3': 'pypy3',
-                   'py': sys.executable}
-for version in '26,27,32,33,34,35,36'.split(','):
+                   'py': sys.executable, 'py2': 'python2', 'py3': 'python3'}
+for version in '26,27,32,33,34,35,36,37'.split(','):
     default_factors['py' + version] = 'python%s.%s' % tuple(version)
 
 hookimpl = pluggy.HookimplMarker("tox")
 
+_dummy = object()
 
-def get_plugin_manager():
+
+def get_plugin_manager(plugins=()):
     # initialize plugin manager
+    import tox.venv
     pm = pluggy.PluginManager("tox")
     pm.add_hookspecs(hookspecs)
     pm.register(tox.config)
     pm.register(tox.interpreters)
+    pm.register(tox.venv)
+    pm.register(tox.session)
     pm.load_setuptools_entrypoints("tox")
+    for plugin in plugins:
+        pm.register(plugin)
     pm.check_pending()
     return pm
 
@@ -141,7 +150,11 @@ class DepOption:
         same package, even if versions differ.
         """
         dep1_name = pkg_resources.Requirement.parse(dep1).project_name
-        dep2_name = pkg_resources.Requirement.parse(dep2).project_name
+        try:
+            dep2_name = pkg_resources.Requirement.parse(dep2).project_name
+        except pkg_resources.RequirementParseError:
+            # we couldn't parse a version, probably a URL
+            return False
         return dep1_name == dep2_name
 
 
@@ -180,7 +193,7 @@ class InstallcmdOption:
         return value
 
 
-def parseconfig(args=None):
+def parseconfig(args=None, plugins=()):
     """
     :param list[str] args: Optional list of arguments.
     :type pkg: str
@@ -188,7 +201,7 @@ def parseconfig(args=None):
     :raise SystemExit: toxinit file is not found
     """
 
-    pm = get_plugin_manager()
+    pm = get_plugin_manager(plugins)
 
     if args is None:
         args = sys.argv[1:]
@@ -214,7 +227,14 @@ def parseconfig(args=None):
             if inipath.check():
                 break
         else:
-            feedback("toxini file %r not found" % (basename), sysexit=True)
+            inipath = py.path.local().join('setup.cfg')
+            if not inipath.check():
+                helpoptions = option.help or option.helpini
+                feedback("toxini file %r not found" % (basename),
+                         sysexit=not helpoptions)
+                if helpoptions:
+                    return config
+
     try:
         parseini(config, inipath)
     except tox.exception.InterpreterNotFound:
@@ -241,12 +261,45 @@ class VersionAction(argparse.Action):
         raise SystemExit(0)
 
 
-class CountAction(argparse.Action):
-    def __call__(self, parser, namespace, values, option_string=None):
-        if hasattr(namespace, self.dest):
-            setattr(namespace, self.dest, int(getattr(namespace, self.dest)) + 1)
-        else:
-            setattr(namespace, self.dest, 0)
+class SetenvDict:
+    def __init__(self, dict, reader):
+        self.reader = reader
+        self.definitions = dict
+        self.resolved = {}
+        self._lookupstack = []
+
+    def __contains__(self, name):
+        return name in self.definitions
+
+    def get(self, name, default=None):
+        try:
+            return self.resolved[name]
+        except KeyError:
+            try:
+                if name in self._lookupstack:
+                    raise KeyError(name)
+                val = self.definitions[name]
+            except KeyError:
+                return os.environ.get(name, default)
+            self._lookupstack.append(name)
+            try:
+                self.resolved[name] = res = self.reader._replace(val)
+            finally:
+                self._lookupstack.pop()
+            return res
+
+    def __getitem__(self, name):
+        x = self.get(name, _dummy)
+        if x is _dummy:
+            raise KeyError(name)
+        return x
+
+    def keys(self):
+        return self.definitions.keys()
+
+    def __setitem__(self, name, value):
+        self.definitions[name] = value
+        self.resolved[name] = value
 
 
 @hookimpl
@@ -259,9 +312,9 @@ def tox_addoption(parser):
                         help="show help about options")
     parser.add_argument("--help-ini", "--hi", action="store_true", dest="helpini",
                         help="show help about ini-names")
-    parser.add_argument("-v", nargs=0, action=CountAction, default=0,
-                        dest="verbosity",
-                        help="increase verbosity of reporting output.")
+    parser.add_argument("-v", action='count', dest="verbosity", default=0,
+                        help="increase verbosity of reporting output. -vv mode turns off "
+                        "output redirection for package installation")
     parser.add_argument("--showconfig", action="store_true",
                         help="show configuration information for all environments. ")
     parser.add_argument("-l", "--listenvs", action="store_true",
@@ -315,15 +368,42 @@ def tox_addoption(parser):
                         help="override sitepackages setting to True in all envs")
     parser.add_argument("--skip-missing-interpreters", action="store_true",
                         help="don't fail tests for missing interpreters")
+    parser.add_argument("--workdir", action="store",
+                        dest="workdir", metavar="PATH", default=None,
+                        help="tox working directory")
 
     parser.add_argument("args", nargs="*",
                         help="additional arguments available to command positional substitution")
 
-    # add various core venv interpreter attributes
-
     parser.add_testenv_attribute(
         name="envdir", type="path", default="{toxworkdir}/{envname}",
-        help="venv directory")
+        help="set venv directory -- be very careful when changing this as tox "
+             "will remove this directory when recreating an environment")
+
+    # add various core venv interpreter attributes
+    def setenv(testenv_config, value):
+        setenv = value
+        config = testenv_config.config
+        if "PYTHONHASHSEED" not in setenv and config.hashseed is not None:
+            setenv['PYTHONHASHSEED'] = config.hashseed
+        return setenv
+
+    parser.add_testenv_attribute(
+        name="setenv", type="dict_setenv", postprocess=setenv,
+        help="list of X=Y lines with environment variable settings")
+
+    def basepython_default(testenv_config, value):
+        if value is None:
+            for f in testenv_config.factors:
+                if f in default_factors:
+                    return default_factors[f]
+            return sys.executable
+        return str(value)
+
+    parser.add_testenv_attribute(
+        name="basepython", type="string", default=None, postprocess=basepython_default,
+        help="executable name or path of interpreter used to create a "
+             "virtual test environment.")
 
     parser.add_testenv_attribute(
         name="envtmpdir", type="path", default="{envdir}/tmp",
@@ -333,15 +413,9 @@ def tox_addoption(parser):
         name="envlogdir", type="path", default="{envdir}/log",
         help="venv log directory")
 
-    def downloadcache(testenv_config, value):
-        if value:
-            # env var, if present, takes precedence
-            downloadcache = os.environ.get("PIP_DOWNLOAD_CACHE", value)
-            return py.path.local(downloadcache)
-
     parser.add_testenv_attribute(
-        name="downloadcache", type="string", default=None, postprocess=downloadcache,
-        help="(deprecated) set PIP_DOWNLOAD_CACHE.")
+        name="downloadcache", type="string", default=None,
+        help="(ignored) has no effect anymore, pip-8 uses local caching by default")
 
     parser.add_testenv_attribute(
         name="changedir", type="path", default="{toxinidir}",
@@ -369,24 +443,13 @@ def tox_addoption(parser):
         name="recreate", type="bool", default=False, postprocess=recreate,
         help="always recreate this test environment.")
 
-    def setenv(testenv_config, value):
-        setenv = value
-        config = testenv_config.config
-        if "PYTHONHASHSEED" not in setenv and config.hashseed is not None:
-            setenv['PYTHONHASHSEED'] = config.hashseed
-        return setenv
-
-    parser.add_testenv_attribute(
-        name="setenv", type="dict", postprocess=setenv,
-        help="list of X=Y lines with environment variable settings")
-
     def passenv(testenv_config, value):
         # Flatten the list to deal with space-separated values.
         value = list(
             itertools.chain.from_iterable(
                 [x.split(' ') for x in value]))
 
-        passenv = set(["PATH", "PIP_INDEX_URL", "LANG"])
+        passenv = set(["PATH", "PIP_INDEX_URL", "LANG", "LD_LIBRARY_PATH"])
 
         # read in global passenv settings
         p = os.environ.get("TOX_TESTENV_PASSENV", None)
@@ -401,6 +464,7 @@ def tox_addoption(parser):
             passenv.add("SYSTEMDRIVE")  # needed for pip6
             passenv.add("SYSTEMROOT")   # needed for python's crypto module
             passenv.add("PATHEXT")      # needed for discovering executables
+            passenv.add("COMSPEC")      # needed for distutils cygwincompiler
             passenv.add("TEMP")
             passenv.add("TMP")
         else:
@@ -453,26 +517,30 @@ def tox_addoption(parser):
         name="usedevelop", type="bool", postprocess=develop, default=False,
         help="install package in develop/editable mode")
 
-    def basepython_default(testenv_config, value):
-        if value is None:
-            for f in testenv_config.factors:
-                if f in default_factors:
-                    return default_factors[f]
-            return sys.executable
-        return str(value)
+    parser.add_testenv_attribute_obj(InstallcmdOption())
 
     parser.add_testenv_attribute(
-        name="basepython", type="string", default=None, postprocess=basepython_default,
-        help="executable name or path of interpreter used to create a "
-             "virtual test environment.")
+        name="list_dependencies_command",
+        type="argv",
+        default="pip freeze",
+        help="list dependencies for a virtual environment")
 
-    parser.add_testenv_attribute_obj(InstallcmdOption())
     parser.add_testenv_attribute_obj(DepOption())
 
     parser.add_testenv_attribute(
         name="commands", type="argvlist", default="",
         help="each line specifies a test command and can use substitution.")
 
+    parser.add_testenv_attribute(
+        "ignore_outcome", type="bool", default=False,
+        help="if set to True a failing result of this testenv will not make "
+             "tox fail, only a warning will be produced")
+
+    parser.add_testenv_attribute(
+        "extras", type="line-list",
+        help="list of extras to install with the source distribution or "
+             "develop install")
+
 
 class Config(object):
     """ Global Tox config object. """
@@ -507,8 +575,7 @@ class TestenvConfig:
         self.factors = factors
         self._reader = reader
 
-    @property
-    def envbindir(self):
+    def get_envbindir(self):
         """ path to directory where scripts/binaries reside. """
         if (sys.platform == "win32"
                 and "jython" not in self.basepython
@@ -517,8 +584,16 @@ class TestenvConfig:
         else:
             return self.envdir.join("bin")
 
+    @property
+    def envbindir(self):
+        return self.get_envbindir()
+
     @property
     def envpython(self):
+        """ path to python executable. """
+        return self.get_envpython()
+
+    def get_envpython(self):
         """ path to python/jython executable. """
         if "jython" in str(self.basepython):
             name = "jython"
@@ -526,12 +601,10 @@ class TestenvConfig:
             name = "python"
         return self.envbindir.join(name)
 
-    # no @property to avoid early calling (see callable(subst[key]) checks)
-    def envsitepackagesdir(self):
+    def get_envsitepackagesdir(self):
         """ return sitepackagesdir of the virtualenv environment.
         (only available during execution, not parsing)
         """
-        self.getsupportedinterpreter()  # for throwing exceptions
         x = self.config.interpreters.get_sitepackagesdir(
             info=self.python_info,
             envdir=self.envdir)
@@ -584,12 +657,18 @@ class parseini:
         self._cfg = py.iniconfig.IniConfig(config.toxinipath)
         config._cfg = self._cfg
         self.config = config
+
+        if inipath.basename == 'setup.cfg':
+            prefix = 'tox'
+        else:
+            prefix = None
         ctxname = getcontextname()
         if ctxname == "jenkins":
-            reader = SectionReader("tox:jenkins", self._cfg, fallbacksections=['tox'])
+            reader = SectionReader("tox:jenkins", self._cfg, prefix=prefix,
+                                   fallbacksections=['tox'])
             distshare_default = "{toxworkdir}/distshare"
         elif not ctxname:
-            reader = SectionReader("tox", self._cfg)
+            reader = SectionReader("tox", self._cfg, prefix=prefix)
             distshare_default = "{homedir}/.tox/distshare"
         else:
             raise ValueError("invalid context")
@@ -604,8 +683,20 @@ class parseini:
 
         reader.addsubstitutions(toxinidir=config.toxinidir,
                                 homedir=config.homedir)
-        config.toxworkdir = reader.getpath("toxworkdir", "{toxinidir}/.tox")
+        # As older versions of tox may have bugs or incompatabilities that
+        # prevent parsing of tox.ini this must be the first thing checked.
         config.minversion = reader.getstring("minversion", None)
+        if config.minversion:
+            minversion = NormalizedVersion(self.config.minversion)
+            toxversion = NormalizedVersion(tox.__version__)
+            if toxversion < minversion:
+                raise tox.exception.MinVersionError(
+                    "tox version is %s, required is at least %s" % (
+                        toxversion, minversion))
+        if config.option.workdir is None:
+            config.toxworkdir = reader.getpath("toxworkdir", "{toxinidir}/.tox")
+        else:
+            config.toxworkdir = config.toxinidir.join(config.option.workdir, abs=True)
 
         if not config.option.skip_missing_interpreters:
             config.option.skip_missing_interpreters = \
@@ -689,10 +780,13 @@ class parseini:
         vc = TestenvConfig(config=config, envname=name, factors=factors, reader=reader)
         reader.addsubstitutions(**subs)
         reader.addsubstitutions(envname=name)
+        reader.addsubstitutions(envbindir=vc.get_envbindir,
+                                envsitepackagesdir=vc.get_envsitepackagesdir,
+                                envpython=vc.get_envpython)
 
         for env_attr in config._testenv_attr:
             atype = env_attr.type
-            if atype in ("bool", "path", "string", "dict", "argv", "argvlist"):
+            if atype in ("bool", "path", "string", "dict", "dict_setenv", "argv", "argvlist"):
                 meth = getattr(reader, "get" + atype)
                 res = meth(env_attr.name, env_attr.default)
             elif atype == "space-separated-list":
@@ -709,9 +803,6 @@ class parseini:
             if atype == "path":
                 reader.addsubstitutions(**{env_attr.name: res})
 
-            if env_attr.name == "install_command":
-                reader.addsubstitutions(envbindir=vc.envbindir, envpython=vc.envpython,
-                                        envsitepackagesdir=vc.envsitepackagesdir)
         return vc
 
     def _getenvdata(self, reader):
@@ -792,24 +883,24 @@ class IndexServerConfig:
 is_section_substitution = re.compile("{\[[^{}\s]+\]\S+?}").match
 
 
-RE_ITEM_REF = re.compile(
-    r'''
-    (?<!\\)[{]
-    (?:(?P<sub_type>[^[:{}]+):)?    # optional sub_type for special rules
-    (?P<substitution_value>[^{}]*)  # substitution key
-    [}]
-    ''',
-    re.VERBOSE)
-
-
 class SectionReader:
-    def __init__(self, section_name, cfgparser, fallbacksections=None, factors=()):
-        self.section_name = section_name
+    def __init__(self, section_name, cfgparser, fallbacksections=None,
+                 factors=(), prefix=None):
+        if prefix is None:
+            self.section_name = section_name
+        else:
+            self.section_name = "%s:%s" % (prefix, section_name)
         self._cfg = cfgparser
         self.fallbacksections = fallbacksections or []
         self.factors = factors
         self._subs = {}
         self._subststack = []
+        self._setenv = None
+
+    def get_environ_value(self, name):
+        if self._setenv is None:
+            return os.environ.get(name)
+        return self._setenv.get(name)
 
     def addsubstitutions(self, _posargs=None, **kw):
         self._subs.update(kw)
@@ -829,17 +920,26 @@ class SectionReader:
         return [x.strip() for x in s.split(sep) if x.strip()]
 
     def getdict(self, name, default=None, sep="\n"):
-        s = self.getstring(name, None)
-        if s is None:
+        value = self.getstring(name, None)
+        return self._getdict(value, default=default, sep=sep)
+
+    def getdict_setenv(self, name, default=None, sep="\n"):
+        value = self.getstring(name, None, replace=True, crossonly=True)
+        definitions = self._getdict(value, default=default, sep=sep)
+        self._setenv = SetenvDict(definitions, reader=self)
+        return self._setenv
+
+    def _getdict(self, value, default, sep):
+        if value is None:
             return default or {}
 
-        value = {}
-        for line in s.split(sep):
+        d = {}
+        for line in value.split(sep):
             if line.strip():
                 name, rest = line.split('=', 1)
-                value[name.strip()] = rest.strip()
+                d[name.strip()] = rest.strip()
 
-        return value
+        return d
 
     def getbool(self, name, default=None):
         s = self.getstring(name, default)
@@ -866,7 +966,7 @@ class SectionReader:
     def getargv(self, name, default=""):
         return self.getargvlist(name, default)[0]
 
-    def getstring(self, name, default=None, replace=True):
+    def getstring(self, name, default=None, replace=True, crossonly=False):
         x = None
         for s in [self.section_name] + self.fallbacksections:
             try:
@@ -881,11 +981,7 @@ class SectionReader:
             x = self._apply_factors(x)
 
         if replace and x and hasattr(x, 'replace'):
-            self._subststack.append((self.section_name, name))
-            try:
-                x = self._replace(x)
-            finally:
-                assert self._subststack.pop() == (self.section_name, name)
+            x = self._replace(x, name=name, crossonly=crossonly)
         # print "getstring", self.section_name, name, "returned", repr(x)
         return x
 
@@ -902,85 +998,110 @@ class SectionReader:
         lines = s.strip().splitlines()
         return '\n'.join(filter(None, map(factor_line, lines)))
 
-    def _replace_env(self, match):
-        match_value = match.group('substitution_value')
-        if not match_value:
-            raise tox.exception.ConfigError(
-                'env: requires an environment variable name')
+    def _replace(self, value, name=None, section_name=None, crossonly=False):
+        if '{' not in value:
+            return value
 
-        default = None
-        envkey_split = match_value.split(':', 1)
+        section_name = section_name if section_name else self.section_name
+        self._subststack.append((section_name, name))
+        try:
+            return Replacer(self, crossonly=crossonly).do_replace(value)
+        finally:
+            assert self._subststack.pop() == (section_name, name)
 
-        if len(envkey_split) is 2:
-            envkey, default = envkey_split
-        else:
-            envkey = match_value
 
-        if envkey not in os.environ and default is None:
-            raise tox.exception.ConfigError(
-                "substitution env:%r: unknown environment variable %r" %
-                (envkey, envkey))
+class Replacer:
+    RE_ITEM_REF = re.compile(
+        r'''
+        (?<!\\)[{]
+        (?:(?P<sub_type>[^[:{}]+):)?    # optional sub_type for special rules
+        (?P<substitution_value>(?:\[[^,{}]*\])?[^:,{}]*)  # substitution key
+        (?::(?P<default_value>[^{}]*))?   # default value
+        [}]
+        ''', re.VERBOSE)
 
-        return os.environ.get(envkey, default)
+    def __init__(self, reader, crossonly=False):
+        self.reader = reader
+        self.crossonly = crossonly
 
-    def _substitute_from_other_section(self, key):
-        if key.startswith("[") and "]" in key:
-            i = key.find("]")
-            section, item = key[1:i], key[i + 1:]
-            if section in self._cfg and item in self._cfg[section]:
-                if (section, item) in self._subststack:
-                    raise ValueError('%s already in %s' % (
-                        (section, item), self._subststack))
-                x = str(self._cfg[section][item])
-                self._subststack.append((section, item))
-                try:
-                    return self._replace(x)
-                finally:
-                    self._subststack.pop()
-
-        raise tox.exception.ConfigError(
-            "substitution key %r not found" % key)
-
-    def _replace_substitution(self, match):
-        sub_key = match.group('substitution_value')
-        val = self._subs.get(sub_key, None)
-        if val is None:
-            val = self._substitute_from_other_section(sub_key)
-        if py.builtin.callable(val):
-            val = val()
-        return str(val)
+    def do_replace(self, x):
+        return self.RE_ITEM_REF.sub(self._replace_match, x)
 
     def _replace_match(self, match):
         g = match.groupdict()
+        sub_value = g['substitution_value']
+        if self.crossonly:
+            if sub_value.startswith("["):
+                return self._substitute_from_other_section(sub_value)
+            # in crossonly we return all other hits verbatim
+            start, end = match.span()
+            return match.string[start:end]
+
+        # special case: all empty values means ":" which is os.pathsep
+        if not any(g.values()):
+            return os.pathsep
 
         # special case: opts and packages. Leave {opts} and
         # {packages} intact, they are replaced manually in
         # _venv.VirtualEnv.run_install_command.
-        sub_value = g['substitution_value']
         if sub_value in ('opts', 'packages'):
             return '{%s}' % sub_value
 
-        handlers = {
-            'env': self._replace_env,
-            None: self._replace_substitution,
-        }
         try:
             sub_type = g['sub_type']
         except KeyError:
             raise tox.exception.ConfigError(
                 "Malformed substitution; no substitution type provided")
 
-        try:
-            handler = handlers[sub_type]
-        except KeyError:
-            raise tox.exception.ConfigError("No support for the %s substitution type" % sub_type)
+        if sub_type == "env":
+            return self._replace_env(match)
+        if sub_type is not None:
+            raise tox.exception.ConfigError(
+                "No support for the %s substitution type" % sub_type)
+        return self._replace_substitution(match)
 
-        return handler(match)
+    def _replace_env(self, match):
+        envkey = match.group('substitution_value')
+        if not envkey:
+            raise tox.exception.ConfigError(
+                'env: requires an environment variable name')
 
-    def _replace(self, x):
-        if '{' in x:
-            return RE_ITEM_REF.sub(self._replace_match, x)
-        return x
+        default = match.group('default_value')
+
+        envvalue = self.reader.get_environ_value(envkey)
+        if envvalue is None:
+            if default is None:
+                raise tox.exception.ConfigError(
+                    "substitution env:%r: unknown environment variable %r "
+                    " or recursive definition." %
+                    (envkey, envkey))
+            return default
+        return envvalue
+
+    def _substitute_from_other_section(self, key):
+        if key.startswith("[") and "]" in key:
+            i = key.find("]")
+            section, item = key[1:i], key[i + 1:]
+            cfg = self.reader._cfg
+            if section in cfg and item in cfg[section]:
+                if (section, item) in self.reader._subststack:
+                    raise ValueError('%s already in %s' % (
+                        (section, item), self.reader._subststack))
+                x = str(cfg[section][item])
+                return self.reader._replace(x, name=item, section_name=section,
+                                            crossonly=self.crossonly)
+
+        raise tox.exception.ConfigError(
+            "substitution key %r not found" % key)
+
+    def _replace_substitution(self, match):
+        sub_key = match.group('substitution_value')
+        val = self.reader._subs.get(sub_key, None)
+        if val is None:
+            val = self._substitute_from_other_section(sub_key)
+        if py.builtin.callable(val):
+            val = val()
+        return str(val)
 
 
 class _ArgvlistReader:
@@ -999,9 +1120,6 @@ class _ArgvlistReader:
         current_command = ""
         for line in value.splitlines():
             line = line.rstrip()
-            i = line.find("#")
-            if i != -1:
-                line = line[:i].rstrip()
             if not line:
                 continue
             if line.endswith("\\"):
@@ -1010,7 +1128,7 @@ class _ArgvlistReader:
             current_command += line
 
             if is_section_substitution(current_command):
-                replaced = reader._replace(current_command)
+                replaced = reader._replace(current_command, crossonly=True)
                 commands.extend(cls.getargvlist(reader, replaced))
             else:
                 commands.append(cls.processcommand(reader, current_command))
@@ -1024,7 +1142,8 @@ class _ArgvlistReader:
 
     @classmethod
     def processcommand(cls, reader, command):
-        posargs = getattr(reader, "posargs", None)
+        posargs = getattr(reader, "posargs", "")
+        posargs_string = list2cmdline([x for x in posargs if x])
 
         # Iterate through each word of the command substituting as
         # appropriate to construct the new command string. This
@@ -1033,18 +1152,18 @@ class _ArgvlistReader:
         newcommand = ""
         for word in CommandParser(command).words():
             if word == "{posargs}" or word == "[]":
-                if posargs:
-                    newcommand += " ".join(posargs)
+                newcommand += posargs_string
                 continue
             elif word.startswith("{posargs:") and word.endswith("}"):
                 if posargs:
-                    newcommand += " ".join(posargs)
+                    newcommand += posargs_string
                     continue
                 else:
                     word = word[9:-1]
             new_arg = ""
             new_word = reader._replace(word)
             new_word = reader._replace(new_word)
+            new_word = new_word.replace('\\{', '{').replace('\\}', '}')
             new_arg += new_word
             newcommand += new_arg
 
@@ -1053,9 +1172,7 @@ class _ArgvlistReader:
         shlexer = shlex.shlex(newcommand, posix=True)
         shlexer.whitespace_split = True
         shlexer.escape = ''
-        shlexer.commenters = ''
-        argv = list(shlexer)
-        return argv
+        return list(shlexer)
 
 
 class CommandParser(object):
