@@ -2,31 +2,31 @@ import sys
 import py
 import re
 import inspect
+from tox import hookimpl
+
 
 class Interpreters:
-    def __init__(self):
+    def __init__(self, hook):
         self.name2executable = {}
         self.executable2info = {}
+        self.hook = hook
 
-    def get_executable(self, name):
+    def get_executable(self, envconfig):
         """ return path object to the executable for the given
         name (e.g. python2.6, python2.7, python etc.)
         if name is already an existing path, return name.
         If an interpreter cannot be found, return None.
         """
         try:
-            return self.name2executable[name]
+            return self.name2executable[envconfig.envname]
         except KeyError:
-            self.name2executable[name] = e = find_executable(name)
-            return e
+            exe = self.hook.tox_get_python_executable(envconfig=envconfig)
+            self.name2executable[envconfig.envname] = exe
+            return exe
 
-    def get_info(self, name=None, executable=None):
-        if name is None and executable is None:
-            raise ValueError("need to specify name or executable")
-        if name:
-            if executable is not None:
-                raise ValueError("cannot specify both name, executable")
-            executable = self.get_executable(name)
+    def get_info(self, envconfig):
+        executable = self.get_executable(envconfig)
+        name = envconfig.basepython
         if not executable:
             return NoInterpreterInfo(name=name)
         try:
@@ -42,20 +42,21 @@ class Interpreters:
         envdir = str(envdir)
         try:
             res = exec_on_interpreter(info.executable,
-                [inspect.getsource(sitepackagesdir),
-                 "print (sitepackagesdir(%r))" % envdir])
+                                      [inspect.getsource(sitepackagesdir),
+                                       "print(sitepackagesdir(%r))" % envdir])
         except ExecFailed:
             val = sys.exc_info()[1]
-            print ("execution failed: %s -- %s" %(val.out, val.err))
+            print("execution failed: %s -- %s" % (val.out, val.err))
             return ""
         else:
             return res["dir"]
 
+
 def run_and_get_interpreter_info(name, executable):
     assert executable
     try:
         result = exec_on_interpreter(executable,
-            [inspect.getsource(pyinfo), "print (pyinfo())"])
+                                     [inspect.getsource(pyinfo), "print(pyinfo())"])
     except ExecFailed:
         val = sys.exc_info()[1]
         return NoInterpreterInfo(name, executable=val.executable,
@@ -63,6 +64,7 @@ def run_and_get_interpreter_info(name, executable):
     else:
         return InterpreterInfo(name, executable, **result)
 
+
 def exec_on_interpreter(executable, source):
     if isinstance(source, list):
         source = "\n".join(source)
@@ -80,6 +82,7 @@ def exec_on_interpreter(executable, source):
                          "could not decode %r" % out)
     return result
 
+
 class ExecFailed(Exception):
     def __init__(self, executable, source, out, err):
         self.executable = executable
@@ -87,21 +90,25 @@ class ExecFailed(Exception):
         self.out = out
         self.err = err
 
+
 class InterpreterInfo:
     runnable = True
 
-    def __init__(self, name, executable, version_info):
+    def __init__(self, name, executable, version_info, sysplatform):
         assert executable and version_info
         self.name = name
         self.executable = executable
         self.version_info = version_info
+        self.sysplatform = sysplatform
 
     def __str__(self):
         return "<executable at %s, version_info %s>" % (
-                self.executable, self.version_info)
+            self.executable, self.version_info)
+
 
 class NoInterpreterInfo:
     runnable = False
+
     def __init__(self, name, executable=None,
                  out=None, err="not found"):
         self.name = name
@@ -117,30 +124,14 @@ class NoInterpreterInfo:
             return "<executable not found for: %s>" % self.name
 
 if sys.platform != "win32":
-    def find_executable(name):
-        return py.path.local.sysfind(name)
+    @hookimpl
+    def tox_get_python_executable(envconfig):
+        return py.path.local.sysfind(envconfig.basepython)
 
 else:
-    # Exceptions to the usual windows mapping
-    win32map = {
-            'python': sys.executable,
-            'jython': "c:\jython2.5.1\jython.bat",
-    }
-    def locate_via_py(v_maj, v_min):
-        ver = "-%s.%s" % (v_maj, v_min)
-        script = "import sys; print(sys.executable)"
-        py_exe = py.path.local.sysfind('py')
-        if py_exe:
-            try:
-                exe = py_exe.sysexec(ver, '-c', script).strip()
-            except py.process.cmdexec.Error:
-                exe = None
-            if exe:
-                exe = py.path.local(exe)
-                if exe.check():
-                    return exe
-
-    def find_executable(name):
+    @hookimpl
+    def tox_get_python_executable(envconfig):
+        name = envconfig.basepython
         p = py.path.local.sysfind(name)
         if p:
             return p
@@ -161,9 +152,32 @@ else:
         if m:
             return locate_via_py(*m.groups())
 
+    # Exceptions to the usual windows mapping
+    win32map = {
+        'python': sys.executable,
+        'jython': "c:\jython2.5.1\jython.bat",
+    }
+
+    def locate_via_py(v_maj, v_min):
+        ver = "-%s.%s" % (v_maj, v_min)
+        script = "import sys; print(sys.executable)"
+        py_exe = py.path.local.sysfind('py')
+        if py_exe:
+            try:
+                exe = py_exe.sysexec(ver, '-c', script).strip()
+            except py.process.cmdexec.Error:
+                exe = None
+            if exe:
+                exe = py.path.local(exe)
+                if exe.check():
+                    return exe
+
+
 def pyinfo():
     import sys
-    return dict(version_info=tuple(sys.version_info))
+    return dict(version_info=tuple(sys.version_info),
+                sysplatform=sys.platform)
+
 
 def sitepackagesdir(envdir):
     from distutils.sysconfig import get_python_lib
